//Copyright (c) 1998 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

package ORG.opengroup.axtp;
import java.beans.*;
import java.net.*;
import java.io.*;
import ORG.opengroup.misc.*;

/**
 * A bean for accessing a Datagram Sockett.
 */
public class DGSocket implements Serializable
{
	private String serializationBeanName="";
	private Serialization serialization=null;
	private int port=0;
	private int inputBufferSize=0;

	transient private DatagramSocket ds=null;
	transient private ClassLoader cl=null;

	/**
	 * Throws an error if not called during DesignTime.
	 */
	public void setSerializationBeanName(String v)
		throws ClassNotFoundException, IOException
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		serializationBeanName=v;
		serialization=(Serialization)Beans.instantiate(null,serializationBeanName);
	}

	public String getSerializationBeanName()
	{
		return serializationBeanName;
	}

	public Serialization getSerialization()
	{
		return serialization;
	}

	public void setPort(int v)
	{
		port=v;
	}

	public int getPort()
	{
		return port;
	}

	public void setInputBufferSize(int v)
	{
		inputBufferSize=v;
	}

	public int getInputBufferSize()
	{
		return inputBufferSize;
	}

	public DGSocket()
	{
		if (!Beans.isDesignTime())
	            init();
	}

	private void writeObject(ObjectOutputStream s)
		throws IOException
	{
		s.defaultWriteObject();
	}

	private void readObject(ObjectInputStream s)
		throws ClassNotFoundException, IOException 
	{
		s.defaultReadObject();
		if (!Beans.isDesignTime())
			init();
	}

	private void init()
	{
		try
		{
			ds=new DatagramSocket(port);
			cl=null;
		} catch (SocketException se)
		{
			throw new Error("Unable to open DatagramSocket on port "+port);
		}
	}

	public void setClassLoader(ClassLoader cl)
	{
		this.cl=cl;
	}

	public void send(Object o, InetAddress ia, int port)
		throws IOException
	{
		byte[] b=serialization.objectToBytes(o);
		send(new DatagramPacket(b,b.length,ia,port));
	}

	public void send(DatagramPacket dp)
		throws IOException
	{
		ds.send(dp);
	}

	public DatagramPacket receive()
		throws IOException
	{
		byte b[]=new byte[inputBufferSize];
		DatagramPacket dp=new DatagramPacket(b,b.length);
		ds.receive(dp);
		return dp;
	}

	public void close()
	{
		ds.close();
	}
}
