//Copyright (c) 1998 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

package ORG.opengroup.axtp.tp;
import ORG.opengroup.axtp.*;
import ORG.opengroup.axtp.transaction.*;
import ORG.opengroup.axtp.tid.*;
import ORG.opengroup.misc.*;
import java.beans.*;
import java.io.*;
import java.net.*;

public class TPImpl implements TP, Runnable
{
	private String dgSocketBeanName="";
	private String tidFactoryBeanName="";
	private String tcsBeanName="";
	private String serverFactoryBeanName="";
	private String retryDelayBeanName="";
	private long transactionRetention=1000*60*60*24*7; //1 week retention default
	private long maxTransactionDuration=1000*60*60; //1 hour maximum transaction duration default
	private long maxClockDelta=1000*60; //1 minute maximum clock difference default
	private boolean noisy=false;
	private DGSocket dgSocket=null;
	private TIDFactory tidFactory=null;
	private TCS tcs=null;
	private RetryDelay retryDelay=null;
	private TransactionFactory transactionFactory=null;
	private ServerFactory serverFactory=null;

	/**
	 * Throws an error if not called during DesignTime.
	 */
	public void setDGSocketBeanName(String v)
		throws ClassNotFoundException, IOException
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		dgSocketBeanName=v;
		dgSocket=(DGSocket)Beans.instantiate(null,dgSocketBeanName);
		transactionFactory=(TransactionFactory)dgSocket.getSerialization();
	}

	public String getDGSocketBeanName()
	{
		return dgSocketBeanName;
	}

	/**
	 * Throws an error if not called during DesignTime.
	 */
	public void setTIDFactoryBeanName(String v)
		throws ClassNotFoundException, IOException
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		tidFactoryBeanName=v;
		tidFactory=(TIDFactory)Beans.instantiate(null,tidFactoryBeanName);
	}

	public String getTIDFactoryBeanName()
	{
		return tidFactoryBeanName;
	}

	/**
	 * Throws an error if not called during DesignTime.
	 */
	public void setTCSBeanName(String v)
		throws ClassNotFoundException, IOException
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		tcsBeanName=v;
		tcs=(TCS)Beans.instantiate(null,tcsBeanName);
	}

	public String getTCSBeanName()
	{
		return tcsBeanName;
	}

	/**
	 * Throws an error if not called during DesignTime.
	 */
	public void setServerFactoryBeanName(String v)
		throws ClassNotFoundException, IOException
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		serverFactoryBeanName=v;
		serverFactory=(ServerFactory)Beans.instantiate
			(null,serverFactoryBeanName);
	}

	public String getServerFactoryBeanName()
	{
		return serverFactoryBeanName;
	}

	/**
	 * Throws an error if not called during DesignTime.
	 */
	public void setRetryDelayBeanName(String v)
		throws ClassNotFoundException, IOException
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		retryDelayBeanName=v;
		retryDelay=(RetryDelay)Beans.instantiate(null,retryDelayBeanName);
	}

	public String getRetryDelayBeanName()
	{
		return retryDelayBeanName;
	}

	public void setTransactionRetention(long v)
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		transactionRetention=v;
	}

	public long getTransactionRetention()
	{
		return transactionRetention;
	}

	public void setMaxTransactionDuration(long v)
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		maxTransactionDuration=v;
	}

	public long getMaxTransactionDuration()
	{
		return maxTransactionDuration;
	}

	public void setMaxClockDelta(long v)
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		maxClockDelta=v;
	}

	public long getMaxClockDelta()
	{
		return maxClockDelta;
	}

	public void setNoisy(boolean v)
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		noisy=v;
	}

	public boolean getNoisy()
	{
		return noisy;
	}

	public boolean isNoisy()
	{
		return noisy;
	}

	public TPImpl()
	{
		if (!Beans.isDesignTime())
	            init();
	}

	private void writeObject(ObjectOutputStream s)
		throws IOException
	{
		s.defaultWriteObject();
	}

	private void readObject(ObjectInputStream s)
		throws ClassNotFoundException, IOException 
	{
		s.defaultReadObject();
		if (!Beans.isDesignTime())
			init();
	}

	private void init()
	{
		tcs.init(this);
		Thread t=new Thread(this);
		t.start();
	}

	public void run()
	{
		while(true)
		{
			try
			{
				DatagramPacket dp=dgSocket.receive();
				Object o=dgSocket.getSerialization().bytesToObject(dp.getData());
				if (o instanceof TransactionObject)
				{
					TransactionObject to=(TransactionObject)o;
					to.setAddress(dp.getAddress());
					to.setPort(dp.getPort());
					byte[] tid=to.getTID();
					if (!(to instanceof TransactionFromClient))
					{
						TransactionContext tc=getClientTC(tid);
						if (tc!=null)
							tc.apply(to);
						else if (isNoisy())
							System.out.println("TPImpl.run from server & no tc "+to);
					}
					else
					{
						TransactionServer ts=getTransactionServer(tid);
						if (ts!=null)
							ts.apply(to);
						else if (to instanceof X)
						{
							long clockDelta=System.currentTimeMillis()
								- TID.getTimeStamp(tid);
							if (clockDelta<0)
								clockDelta=-clockDelta;
							if (clockDelta<maxClockDelta)
							{
								X x=(X)to;
								long transactionDuration=x.getTransactionDuration();
								if (transactionDuration<maxTransactionDuration)
								{
									ts=serverFactory.createTransactionServer();
									ts.setClientAddress(to.getAddress());
									ts.setClientPort(to.getPort());
									ts.setTID(tid);
									boolean primary=x.getPrimary();
									ts.setPrimary(primary);
									ts.init(this);
									if (primary)
									{
										addTC(ts,transactionRetention);
										ts.setDuration(x.getTransactionDuration());
									}
									else
										addTC(ts,0);
									ts.apply(x);
								}
								else if (isNoisy())
									System.out.println("TPImpl.run exceeds duration");
							}
							else if (isNoisy())
								System.out.println("TPImpl.run max delta exceeded: "+clockDelta);
						}
						else if (isNoisy())
							System.out.println("TPImpl.run not X");
					}
				}
				else if (isNoisy())
					System.out.println("TPImpl.run not to");
			} catch (Exception e)
			{
				if (isNoisy())
				{
					System.out.println("TPImpl.run Exception="+e.toString());
					e.printStackTrace();
				}
			}
		}
	}

	public void send(PacketObject po)
	{
		try
		{
			dgSocket.send(po,po.getAddress(),po.getPort());
		} catch (IOException e)
		{
		}
	}

	public byte[] createTID()
	{
		return tidFactory.createTID();
	}

	public void addTC(TransactionContext tc, long transactionRetention)
	{
		tcs.addTC(tc,transactionRetention);
	}

	public TransactionServer getTransactionServer(byte[] tid)
	{
		return tcs.getTransactionServer(tid);
	}

	public TransactionContext getClientTC(byte[] tid)
	{
		return tcs.getClientTC(tid);
	}

	public long setTimer(TransactionContext tc, long context, long delay)
	{
		return tcs.setTimer(tc,context,delay);
	}
	
	public void clearTimer(TransactionContext tc, long context)
	{
		tcs.clearTimer(tc,context);
	}
	
	public void clearAllTimers(TransactionContext tc)
	{
		tcs.clearAllTimers(tc);
	}

	public void deleteTC(TransactionContext tc)
	{
		tcs.deleteTC(tc);
	}

	public ExecutionContext createServerXC(X x, TransactionServer ts)
		throws ClassNotFoundException, IOException
	{
		ExecutionContext serverXC=serverFactory.createExecutionContext();
		serverXC.setTransactionServer(ts);
		serverXC.setXID(x.getXID());
		return serverXC;
	}

	public RetryDelay cloneRetryDelay()
	{
		return (RetryDelay)retryDelay.clone();
	}

	public TransactionFactory getTransactionFactory()
	{
		return transactionFactory;
	}
}
