//Copyright (c) 1998 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

package ORG.opengroup.axtp.tp;
import ORG.opengroup.axtp.*;
import ORG.opengroup.axtp.tid.*;
import java.util.*;
import java.beans.*;
import java.io.*;

/**
 * Transient Transaction Context Set
 */
public class TransientTCS implements TCS, Runnable
{
	private long interval=100;

	private transient TP tp;
	private transient Hashtable cht=null;
	private transient Hashtable sht=null;
	private transient Vector timers=null;

	public void setInterval(long v)
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		interval=v;
	}

	public long getInterval()
	{
		return interval;
	}

	public void init(TP tp)
	{
		this.tp=tp;
		cht=new Hashtable();
		sht=new Hashtable();
		timers=new Vector();
		Thread t=new Thread(this);
		t.start();
	}

	public void addTC(TransactionContext tc, long retention)
	{
		if (tc instanceof TransactionServer)
			sht.put(new TID(tc.getTID()),tc);
		else
			cht.put(new TID(tc.getTID()),tc);
		if (retention>0)
			setTimer(tc,TP.RETENTION_TIMER,retention);
	}

	public TransactionServer getTransactionServer(byte[] tid)
	{
		return (TransactionServer)sht.get(new TID(tid));
	}

	public TransactionContext getClientTC(byte[] tid)
	{
		return (TransactionContext)cht.get(new TID(tid));
	}

	public void apply(TransactionContext tc, TransactionObject to)
	{
		tc.apply(to);
	}

	public long setTimer(TransactionContext tc, long timerContext, long delay)
	{
		clearTimer(tc,timerContext);
		long when=delay+System.currentTimeMillis();
		timers.addElement(new Timer(tc,timerContext,when));
		return when;
	}

	public void clearTimer(TransactionContext tc, long timerContext)
	{
		synchronized(timers)
		{
			int i;
			for (i=0;i<timers.size();++i)
			{
				Timer timer=(Timer)timers.elementAt(i);
				if (tc==timer.tc && timerContext==timer.context)
				{
					timers.removeElementAt(i);
					return;
				}
			}
		}
	}

	public void clearAllTimers(TransactionContext tc)
	{
		synchronized(timers)
		{
			int i;
			for (i=0;i<timers.size();++i)
			{
				Timer timer=(Timer)timers.elementAt(i);
				if (tc==timer.tc)
					timers.removeElementAt(i);
			}
		}
	}

	public void deleteTC(TransactionContext tc)
	{
		TID tid=new TID(tc.getTID());
		if (tc instanceof TransactionServer)
			sht.remove(tid);
		else
			cht.remove(tid);
		clearAllTimers(tc);
	}

	public void run()
	{
		while(true)
		{
			int i;
			long t=System.currentTimeMillis();
			for (i=0;i<timers.size();)
			{
				Timer timer;
				synchronized(timers)
				{
					timer=(Timer)timers.elementAt(i);
					if (t>timer.when)
					{
						timers.removeElementAt(i);
					}
					else
					{
						++i;
						timer=null;
					}
				}
				if (timer!=null)
				{
					if (timer.context!=TP.RETENTION_TIMER)
					{
						timer.tc.timeout(timer.context);
					}
					else
					{
						deleteTC(timer.tc);
					}
				}
			}
			try
			{
				Thread.sleep(interval);
			} catch (InterruptedException e)
			{
				throw new Error(e.toString());
			}
		}
	}

	private class Timer
	{
		TransactionContext tc;
		long context;
		long when;

		Timer(TransactionContext tc, long context, long when)
		{
			this.tc=tc;
			this.context=context;
			this.when=when;
		}
	}
}
