import java.net.*;
import ORG.opengroup.axtp.*;
import ORG.opengroup.axtp.tp.*;
import ORG.opengroup.axtp.transaction.*;

public class Client implements TransactionContext
{
	private byte[] tid;
	private InetAddress serverAddress;
	private int serverPort=0;
	private RetryDelay retryDelay;
	private X x=null;
	private ClientDecision cd=null;
	private long xid=1;
	private long soj=0;
	private boolean active=false;

	private transient TP tp=null;

	public Client()
		throws UnknownHostException
	{
		serverAddress=InetAddress.getLocalHost();
	}

	public void setServerAddress(InetAddress serverAddress)
	{
		this.serverAddress=serverAddress;
	}

	public InetAddress getServerAddress()
	{
		return serverAddress;
	}

	public void setServerPort(int serverPort)
	{
		this.serverPort=serverPort;
	}

	public int getServerPort()
	{
		return serverPort;
	}

	public void setTID(byte[] tid)
	{
		this.tid=tid;
	}

	public byte[] getTID()
	{
		return tid;
	}

	public void init(TP tp)
	{
		this.tp=tp;
	}

	public synchronized void apply(TransactionObject to)
	{
		if (to instanceof A)
		{
			if (!active)
				return;
			A a=(A)to;
			long id=a.getXID();
			if (xid!=id)
			{
				if ((id+1)==xid)
				{
					tp.setTimer(this,xid,retryDelay.getCurrentDelay());
					tp.send(x);
				}
				return;
			}
			tp.clearTimer(this,xid);
			if (xid==1000)
			{
				active=false;
				cd=tp.getTransactionFactory().createC();
				retryDelay.resetDelay();
				cd.setAddress(serverAddress);
				cd.setPort(serverPort);
				cd.setTID(tid);
				tp.send(cd);
				return;
			}
			++xid;
			retryDelay.resetDelay();
			x=tp.getTransactionFactory().createX();
			x.setAddress(serverAddress);
			x.setPort(serverPort);
			x.setTID(tid);
			x.setXID(xid);
			x.setPrimary(true);
			x.setTransactionDuration(1000*60);
			x.setZID(xid-1);
			tp.setTimer(this,xid,retryDelay.getCurrentDelay());
			tp.send(x);
		}
		else if (to instanceof N)
		{
			active=false;
			cd=tp.getTransactionFactory().createR();
			retryDelay.resetDelay();
			cd.setAddress(serverAddress);
			cd.setPort(serverPort);
			cd.setTID(tid);
			tp.send(cd);
		}
		else if (to instanceof W)
		{
			long delta=System.currentTimeMillis()-soj;
			System.out.println("Failure on xid "+xid+" after "+delta+" millis.");
			tp.deleteTC(this);
		}
		else if (to instanceof Q)
		{
			long delta=System.currentTimeMillis()-soj;
			System.out.println("Success on xid "+xid+" after "+delta+" millis.");
			tp.deleteTC(this);
		}
	}

	public synchronized void timeout(long context)
	{
		System.out.println("Client.timeout, context="+context);
		if (active && context!=xid)
			return;
		if (!active && context!=-999)
			return;
		retryDelay.incrimentDelay();
		tp.setTimer(this,xid,retryDelay.getCurrentDelay());
		if (active)
			tp.send(x);
		else
			tp.send(cd);
	}

	public void begin(TP tp)
	{
		init(tp);
		tid=tp.createTID();
		tp.addTC(this,0);
		retryDelay=tp.cloneRetryDelay();
		retryDelay.resetDelay();
		x=tp.getTransactionFactory().createX();
		x.setAddress(serverAddress);
		x.setPort(serverPort);
		x.setTID(tid);
		x.setXID(xid);
		x.setPrimary(true);
		x.setTransactionDuration(1000*60);
		active=true;
		tp.setTimer(this,xid,retryDelay.getCurrentDelay());
		System.out.println("sending first");
		soj=System.currentTimeMillis();
		tp.send(x);
	}
}
